"""TUI screens for Parxy."""

from parxy_cli.tui.screens.folder_selection import FolderSelectionScreen
from parxy_cli.tui.screens.browse import BrowseScreen
from parxy_cli.tui.screens.viewer import ViewerScreen

__all__ = ['FolderSelectionScreen', 'BrowseScreen', 'ViewerScreen']
